/*

        WAIMOTOR.C

        Nucleo Central de The Wailer.
	Version 2.1.1.

        (C) 1993/1995 Angel Ortega.


        Variables:

        int Conferencia;
        long Mensaje;
        char Conferencias[MAX_CONFS][MAX_NOM_CONF];
        int NumConfes;
        int NumMsgs;
        int Histo;
        char To[150];
        char Remit[150];
        char Subject[100];
        char Fecha[150];
        int ancho_linea=75;
        char Yo[30]="";
        int use_histo;


        Funciones:

        void SetMsgFlags(unsigned char modo);

	int ReadMsg(char * hptr, int hsize, char * bptr, int bsize,
		void (* procstr)(char * cadena),
		void (* multipart) (void));

        int QuoteMsg(char * bptr, int bsize,
                void (* procstr)(char * cadena));

	int WriteMsg(char * bptr, char * conf, char * objfile,
		char * objdesc);

        int ReadList(void (* newelem)(void));

        int Index(char * dat, char * ind);

        void ProcessNew(void);

        void InitRespFile(void);

        int InitW(void);

        void EndW(void);

*/

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>


#if !defined(O_BINARY)
        #define O_BINARY 0
        #define READ_BINARY "r"
	#define NL_STR "\n"
	#include <unistd.h>
	#define UNIX_HOST
#else
        #define READ_BINARY "rb"
        #include <io.h>
        #include <dir.h>
        #define NL_STR "\r\n"
	#define MSDOS_HOST
#endif


#if !defined(min)
        #define min(x,y) ((x)<(y))?(x):(y)
#endif


/*******************
     Constantes
*******************/

/* ningun mensaje seleccionado */
#define NO_MENSAJE 0xFFFFFFFF

/* caracter(es) de quote */
#define QUOTE_CHARS "> "

/* fichero de conferencias */
#define CONFE_FILE "conferen.dat"
/* mensajes nuevos */
#define NUEVOS_DAT "nuevos.dat"
/* historico de mensajes */
#define HISTO_DAT "historic.dat"
/* indice de nuevos */
#define NUEVOS_IND "nuevos.ind"
/* indice del historico */
#define HISTO_IND "historic.ind"
/* fichero temporal */
#define TMP_FILE "wailer.tmp"
/* fichero con la firma */
#define FIRM_FILE "wailer.fir"
/* fichero con los tags */
#define TAG_FILE "wailer.tag"
/* directorio de ficheros */
#define FILES_DIR "files"
/* fichero con las extensiones MIME */
#define MIME_EXT "mime.dat"

/* mascara de borrado */
#define MASK_BOR 0x80

/* mascara de leido */
#define MASK_LEI 0x40

/* mascara de conferencia */
#define MASK_CONF 0x3F

/* maximo de conferencias */
#define MAX_CONFS 64

/* maximo del nombre de conferencia */
#define MAX_NOM_CONF 40

/* MIME */

/* boundary de los mensajes multipart/mixed MIME */
char * WailerBoundary="___The_Wailer_CORTAR_POR_AQUI___";

/* tabla de caracteres en base64 (MIME) */
char Base64[66]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"\
		"0123456789+/=";

/* tokens MIME */
char * X_MimeContentType="X-Content-Type: ";
char * X_MimeConTraEnc="X-Content-Transfer-Encoding: ";
char * X_MimeConDes="X-Content-Description: ";
char * MimeContentType="Content-Type: ";
char * MimeConTraEnc="Content-Transfer-Encoding: ";
char * MimeConDes="Content-Description: ";


/*************************
         Globales
*************************/

/* nombre del programa (general) */
char * APPGEN="The Wailer";

/* conferencia seleccionada */
int Conferencia=-1;

/* mensaje seleccionado */
long Mensaje=NO_MENSAJE;

/* array con las conferencias */
char Conferencias[MAX_CONFS][MAX_NOM_CONF];

/* numero de conferencias */
int NumConfes=0;

/* numero de mensajes en esta conferencia */
int NumMsgs=0;

/* flag de historico: 1, mensajes viejos; 0, nuevos */
int Histo=0;

/* estructura del indice de mensajes */
struct tagind
{
        unsigned char dat;      /* bits 0-5: numero de conferencia;
                                   bit 6: leido;
                                   bit 7: borrado */
        long offset;            /* offset del principio del mensaje
                                   en el disco */
        long tamcab;    	/* tamano de la cabecera del mensaje */
	long tambody;   	/* tamano del cuerpo del mensaje */
        unsigned int offsub;    /* offset del subject */
        unsigned int offfrom;   /* offset del remitente */
        unsigned int offto;     /* offset del destinatario */
        unsigned int num;       /* numero del mensaje */
};

/* mensaje actual */
struct tagind Msg;

/* destinatario del mensaje */
char To[150];
/* remitente del mensaje */
char Remit[150];
/* subject del mensaje */
char Subject[100];
/* fecha del mensaje */
char Fecha[150];
/* tabla origen del mensaje */
char Tabla[40];
/* gestor de correo origen */
char Mailer[40];
/* content-type del mensaje actual */
char thisContentType[150];
/* content-transfer-encoding del mensaje actual */
char thisConTraEnc[50];
/* boundary en caso de que sea un mensaje MIME */
char thisBoundary[100];
/* descripcin de un objeto */
char thisDescription[250];
/* charset actual */
char thisCharset[40];
/* nombre del fichero grabado */
char thisFileName[30];

/* killfile */
char KillFile[512];

/* linea temporal */
char lineatmp[4096];

/* ancho de la linea en el campo de texto */
int ancho_linea=75;

/* usuario */
char Yo[30]="";

/* fichero de inicializacion (antes era un #define) */
char INI_FILE[80]="wailer.ini";
/* fichero de respuestas */
char RESP_FILE[80]="contesta";
/* fichero de mensajes */
char MSG_FILE[80]="mensajes";

/* estadisticas */
struct
{
        int mensajes;
        int personales;
} Stats;

/* flag que activa el uso del historico; si esta a 0, no se
   guarda nada */
int use_histo=1;



/************************
        Funciones
************************/


/** funciones de cadenas **/

#define IS_NUM(c) ((c)>='0' && (c)<='9')

int StrCmp(char * one, char * two)
/* compara dos cadenas, ignorando mayusculas/minusculas */
{
	int i;
	char a,b;

	for(;;)
	{
        	a=*one++;
		if(a>='a' && a<='z')
			a-=32;

		b=*two++;
		if(b>='a' && b<='z')
			b-=32;

		if(a=='\0' && b=='\0')
                	return(0);

		i=a-b;

		if(i!=0)
			return(i);
	}
}


int StrnCmp(char * one, char * two, int num)
/* compara dos cadenas, ignorando mayusculas/minusculas, num caracteres */
{
	int i;
	char a,b;

	for(;num;num--)
	{
        	a=*one++;
		if(a>='a' && a<='z')
			a-=32;

		b=*two++;
		if(b>='a' && b<='z')
			b-=32;

		if(a=='\0' && b=='\0')
                	return(0);

		i=a-b;

		if(i!=0)
			return(i);
	}

        return(0);
}


int begins(char * org, char * des)
/* compara si org comienza con des */
{
	return(StrnCmp(org, des, strlen(des))); 
}


char * StrLwr(char * str)
/* pasa a minusculas la cadena */
{
	char * ptr;

	for(ptr=str;*ptr;ptr++)
	{
		if(*ptr>='A' && *ptr<='Z')
			(*ptr)+=32;
	}

	return(str);
}


/** funciones de proceso de tokens RFC822 **/

void GetLogName(char * str)
/* coge de la direccion internet solo el logname */
{
	int n,m;
        char tmp[250];

        strcpy(tmp,str);

        for(n=0;tmp[n]!='@' && tmp[n]!='\n' && tmp[n]!='\0';n++);

        if(tmp[n]!='@')
        {
                str[n]='\0';
                return;
        }

        for(m=1,n--;n>0;m++,n--)
        {
                if(tmp[n]==' ' || tmp[n]=='<')
                {
                        n++;
                        m--;
                        break;
                }
        }

        memcpy(str,&tmp[n],m);
        str[m]='\0';
}


int GetTokenValue(char * line, char * token, char * buffer)
/* si line contiene el token, se guarda sobre buffer */
{
	if(begins(line, token)==0)
        {
		strcpy(buffer, line+strlen(token));
		return(1);
	}
	else
		return(0);
}


int GetTokenParam(char * line, char * token, char * buffer)
/* busca en la lnea el parametro token y si esta lo escribe sobre buffer */
{
        char searchfor;
	*buffer='\0';

	while(*line)
	{
		/* busca el ; */
		for(;*line && *line!=';';line++);

		if(*line=='\0')
			return(0);

		/* se salta los posibles espacios */
		line++;

		for(;*line && *line==' ';line++);

		if(*line=='\0')
			return(0);

		/* compara */
		if(begins(line,token)==0)
		{
			line+=strlen(token);

			if(*line=='"')
                        {
				line++;
				searchfor='"';
			}
			else
                        	searchfor=' ';

                        /* copia hasta el final o hasta el siguiente parametro */
			for(;*line && *line!=searchfor;line++,buffer++)
				*buffer=*line;
			*buffer='\0';

                	return(1);
		}
	}

	return(0);
}


/** funciones base64 **/

void EncodeBase64(unsigned char * des, unsigned char * org, int size)
/* codifica en base64 org sobre des */
{
	int n;

	while(size>0)
	{
		n=(*org)>>2;
		*(des++)=Base64[n];		

		if(size>1)
			n=(((*org)<<4)&0x3f)|(((*(org+1))>>4)&0x3f);
		else
                	n=((*org)<<4)&0x3f;

                org++;
		*(des++)=Base64[n];

		size--;

		if(size>1)
			n=(((*org)<<2)&0x3f)|(((*(org+1))>>6)&0x3f);
		else
		if(size==1)
			n=((*org)<<2)&0x3f;
		else
			n=64;

		org++;
		*(des++)=Base64[n];

		size--;

		if(size>=1)
			n=(*org)&0x3f;
		else
			n=64;

		org++;
		*(des++)=Base64[n];

		size--;
	}

	*des='\0';
}	


char SeekBase64(char c)
/* busca c en la tabla de base 64 */
{
	char n;

	if(c=='=')
		return(-2);

	for(n=0;Base64[n];n++)
	{
		if(Base64[n]==c)
			break;
	}

	if(Base64[n]=='\0')
		return(-1);
	else
        	return(n);
}


int DecodeBase64(unsigned char * des, unsigned char * org, int * size)
/* decodifica en base64 org sobre des */
{
	char n;

	*size=0;

	while(*org)
	{
		if((n=SeekBase64(*org))<0)
			break;

                org++;
                *des=(n<<2);

		if((n=SeekBase64(*org))<0)
			return(0);

                org++;
		*des|=(n>>4);
                des++;
		*des=(n<<4);

                (*size)++;

		if((n=SeekBase64(*org))<0)
			return(0);

                org++;
		*des|=(n>>2);
                des++;
		*des=(n<<6);

		(*size)++;

		if((n=SeekBase64(*org))<0)
			return(0);

                org++;
		*des|=n;
		des++;

		(*size)++;
	}

	if(n==-1)
		return(0);
	else
		return(1);
}



int DecodeUudecode(unsigned char * des, unsigned char * org, int * size)
/* decodifica una linea en formato uuencode */
{
#define	DEC(c)	(((c) - ' ') & 077)		/* single character decode */

	int n;
        char ch;

	if((n=DEC(*org))<=0)
		return(0);

	*size=n;

	for(++org;n>0;org+=4,n-=3)
        {
		if(n>=3)
		{
			ch=DEC(org[0])<<2 | DEC(org[1])>>4;
                        *(des++)=ch;
			ch=DEC(org[1])<<4 | DEC(org[2])>>2;
			*(des++)=ch;
			ch=DEC(org[2])<<6 | DEC(org[3]);
			*(des++)=ch;
		}
		else
		{
			if(n>=1)
			{
				ch=DEC(org[0])<<2 | DEC(org[1])>>4;
				*(des++)=ch;
			}
			if (n>=2)
			{
				ch=DEC(org[1])<<4 | DEC(org[2])>>2;
				*(des++)=ch;
			}
			if (n>=3)
			{
				ch=DEC(org[2])<<6 | DEC(org[3]);
				*(des++)=ch;
			}
		}
	}

	return(1);
}


void DecodeQuotPrin(unsigned char * des, unsigned char * org)
/* decodifica en Quoted-Printable */
{
	char tmp[5];
	int c;

	while(*org)
	{
		c=*org;

		if(c=='=')
		{
			org++;

			if(*org=='\r' || *org=='\n' || *org=='\0')
				c=*org;
			else
                        {
				tmp[0]=*org;
				org++;
				tmp[1]=*org;
				tmp[2]='\0';

				sscanf(tmp,"%x",&c);
			}
		}

		*des=c;

		des++;
		org++;
	}

	*des='\0';
}


/** funciones de conferencias */

int LoadConf(void)
/* carga en el array Conferencias las conferencias. Devuelve 0 si no
   se pudo carga el fichero de conferencias */
{
        int i;

        if((i=open(CONFE_FILE, O_RDONLY|O_BINARY))!=-1)
        {
                NumConfes=read(i, Conferencias, MAX_CONFS*MAX_NOM_CONF)
                        /MAX_NOM_CONF;

                close(i);

                return(1);
        }
        else
                return(0);
}


int GetConf(char * conferencia)
/* devuelve el numero asociado a la conferencia. Si no esta en la tabla,
   se genera una nueva y se vuelca la tabla a disco. Si hay algun error
   se devuelve -1 */
{
        int n;
        int o;
        char * ptr;

        /* se salta los prefijos */
	if(memcmp(conferencia,"crcok.",6)==0)
		conferencia+=6;

        /* se recorre las conferencias */
        for(n=0;n<NumConfes;n++)
        {
                if(StrCmp(conferencia, Conferencias[n])==0)
                        break;
        }

        /* si esta, pues ese es el numero */
        if(n!=NumConfes)
                return ((unsigned char) n);

        /* aqui llega cuando aun no existe: hay que crearla */

        /* si ya no caben mas (cosa improbable) */
        if((NumConfes+1)==MAX_CONFS)
                return(-1);

        /* se copia */
        strcpy(Conferencias[NumConfes], conferencia);
        NumConfes++;

        /* se abre (o se crea) el fichero de conferencias */
        if((o=open(CONFE_FILE,O_CREAT|O_TRUNC|O_BINARY|O_WRONLY,
                S_IREAD|S_IWRITE))==-1)
                return(-1);

        /* se escribe el array */
        write(o, Conferencias, NumConfes*MAX_NOM_CONF);

        close(o);

        return((unsigned char) (NumConfes-1));
}


/** funciones de mensaje **/

int GoToIndOffset(void)
/* posiciona el puntero al fichero indice sobre el offset del
   tag del mensaje actual. Devuelve el handle del fichero */
{
        int i;
        int off;

        /* se abre el temporal, que contiene los offsets reales */
        if((i=open(TMP_FILE, O_RDONLY|O_BINARY))!=-1)
        {
                lseek(i, Mensaje*sizeof(int), 0);
                read(i,&off,sizeof(int));
                close(i);

                /* se abre el indice */
                if(Histo)
                        i=open(HISTO_IND, O_RDWR|O_BINARY);
                else
                        i=open(NUEVOS_IND, O_RDWR|O_BINARY);

                /* se posiciona */
                lseek(i, off*sizeof(struct tagind), 0);
        }

        return(i);
}


FILE * GoToDatOffset(void)
/* posiciona el puntero sobre el primer caracter de la cabecera de
   un mensaje. Devuelve un FILE * */
{
        int i;
        FILE * f;

        /* coge el offset del tag */
        if((i=GoToIndOffset())==-1)
                return(NULL);

        read(i,&Msg,sizeof(Msg));
        close(i);

        /* se abre el fichero de datos */
        if(Histo==1)
                f=fopen(HISTO_DAT, READ_BINARY);
        else
                f=fopen(NUEVOS_DAT, READ_BINARY);

	fseek(f, Msg.offset, 0);

	return(f);
}


void SetMsgFlags(unsigned char modo)
/* cambia los flags al mensaje actual. Permite intercambiar el flag
   de borrado. Siempre activa el flag de leido */
{
        int i;

        /* se marca como borrado (si mask es MASK_BOR) */
        if(modo&MASK_BOR)
                Msg.dat^=MASK_BOR;

        /* ya leido */
        Msg.dat|=MASK_LEI;

        /* coge la posicion del tag en el indice */
        if((i=GoToIndOffset())!=-1)
        {
                /* se escribe el tag del mensaje */
                write(i, &Msg, sizeof(Msg));
                close(i);
        }
}


char * Cpy(char * ptr, char * lin, int * size,
        void (* procstr)(char * cadena))
/* ejecuta procstr con lin, copia lin sobre ptr, decrementa size */
{
        int s;

        /* si queda sitio... */
        if((*size) && ptr)
        {
                /* se ejecuta la funcion de procesamiento */
                if(procstr!=NULL)
                        (* procstr)(lin);

                /* copia sobre hptr */
                if((s=strlen(lin))>(*size))
                        s=*size;

                memcpy(ptr, lin, s);
                ptr+=s;
                (*size)-=s;
        }

        return(ptr);
}


void Description(char * filename)
/* crea una linea en el fichero descript.ion */
{
	char tmp[1024];
	FILE * f;

	if((f=fopen("descript.ion", "r+"))==NULL)
        {
		if((f=fopen("descript.ion", "w"))==NULL)
			return;
	}
	else
	{
		for(;;)
                {
			/* lee las lineas */
			if(fgets(tmp, sizeof(tmp), f)==NULL)
				break;

			/* si ya esta, no se guarda nada */
			if(begins(tmp, filename)==0)
			{
				fclose(f);
				return;
			}
		}
	}

	fprintf(f,"%s %s%s", filename, thisDescription, NL_STR);

	fclose(f);
}


void NormFileName(char * filename)
/* normaliza el nombre del fichero, de forma que sea valido */
/* dependiente del sistema operativo */
{
#if defined(MSDOS_HOST)
	char tmp[100];
	int n,m,i;

	if(*filename=='\0')
		return;

	m=0;
	if(filename[m]=='.')
		m++;

	for(n=0;n<8 && filename[m]!='\0' && filename[m]!='.';n++,m++)
		tmp[n]=filename[m];

	if(filename[m]=='.')
	{
		tmp[n++]='.';

		for(m=strlen(filename)-1;filename[m]!='.' && m;m--);
		m++;

		for(i=0;i<3 && filename[m]!='\0';i++,n++,m++)
			tmp[n]=filename[m];
	}

	tmp[n]='\0';

        strcpy(filename, tmp);
#endif
}


int ProcessMsg(char * hptr, int hsize, char * bptr, int bsize,
	void (* procstr)(char * cadena), int quote,
	void (* multipart)(void))
/* carga la cabecera y el cuerpo del mensaje actual.
   Procstr es una funcion que sera llamada cada vez que
   se lea una linea con la linea leida como argumento, pudiendose
   modificar al gusto (p.e. anadir un \r\n, o lo que sea).
   Devuelve 0 si no se cargo nada. multipart es llamada cada
   vez que una de las partes de un mensaje multipart/mixed
   es procesada */
{
        FILE * f;
        int n;
        int i;
	long l;
	int todisk;
        int usequotprin;
	int usebase64;
        int useuuencode=0;
        char decodedline[150];
        int reload;

        if((f=GoToDatOffset())==NULL)
                return(0);

	/* se vacian las cadenas */
        Subject[0]=0;
	Fecha[0]=0;
        thisDescription[0]='\0';
	strcpy(thisContentType, "text/plain; charset=us-ascii");
        strcpy(thisConTraEnc, "8bit");
	strcpy(thisCharset, "us-ascii");

        /* se posiciona sobre la cabecera */

        /* lee la cabecera */
        for(;;)
        {
                if(fgets(lineatmp, sizeof(lineatmp)-1, f)==NULL)
                        break;
                lineatmp[strlen(lineatmp)-1]=0;

                /* si es la ultima de la cabecera */
                if(lineatmp[0]==0)
                        break;

		/* se almacenan los tokens importantes */
		GetTokenValue(lineatmp,"Date: ",Fecha);
		GetTokenValue(lineatmp,"From: ",Remit);
		GetTokenValue(lineatmp,"Subject: ",Subject);
		GetTokenValue(lineatmp,X_MimeContentType,thisContentType);
		GetTokenValue(lineatmp,X_MimeConTraEnc,thisConTraEnc);
		GetTokenValue(lineatmp,MimeContentType,thisContentType);
		GetTokenValue(lineatmp,MimeConTraEnc,thisConTraEnc);

                /* se copia */
                hptr=Cpy(hptr, lineatmp, &hsize, procstr);
        }

        if(hptr)
                *hptr='\0';

        GetLogName(Remit);

	/* decide sobre MIME: */
	/* si contiene un parametro name, se graba como tal */
	if(GetTokenParam(thisContentType,"name=",thisFileName))
        {
		chdir(FILES_DIR);

		todisk=open(thisFileName, O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,
			S_IREAD|S_IWRITE);
	}
	else
	{
		todisk=0;

		/* no es un objeto por si solo (lo mas habitual)
		   ver si es multipart; lo intenta cogiendo el
		   boundary */
                GetTokenParam(thisContentType,"boundary=",thisBoundary);
        }

	/* ver si hay que usar base64 */
	usebase64=!StrCmp(thisConTraEnc,"base64");

	/* ver si hay que usar Quoted-Printable */
	usequotprin=!StrCmp(thisConTraEnc,"quoted-printable");

	if(!GetTokenParam(thisContentType,"charset=",thisCharset))
		strcpy(thisCharset,"us-ascii");

        /* se posiciona sobre el cuerpo */
        fseek(f, (Msg.offset+Msg.tamcab)-1, 0);

        /* se lee el numero de bytes que nos dice el tag */
        reload=0;
        for(l=Msg.tambody;l>0 && bsize>0;)
        {
                /* lee la linea */
                if(fgets(lineatmp, sizeof(lineatmp)-1, f)==NULL)
                        break;
                lineatmp[strlen(lineatmp)-1]=0;

                l-=(strlen(lineatmp)+1);

		/* si se esta recargando... */
		if(reload)
		{
                	/* ha terminado la subcabecera? */
			if(lineatmp[0]=='\0')
                        {
				reload=0;

				thisFileName[0]='\0';
				GetTokenParam(thisContentType,"name=",thisFileName);

				NormFileName(thisFileName);

				/* chequea el name= */
				if(begins(thisContentType,"text/plain")!=0 || thisFileName[0]!='\0')
				{
                                	if(thisFileName[0]=='\0')
						sprintf(thisFileName,"%02X%06X.",Conferencia,Msg.num);

					chdir(FILES_DIR);

					todisk=open(thisFileName,
						O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,
						S_IREAD|S_IWRITE);

					/* aade una marca en el mensaje */
					if(todisk!=-1 && !quote)
                                        {
						sprintf(lineatmp,"%s** Objeto MIME: %s%s",
							NL_STR,thisFileName,NL_STR);
						bptr=Cpy(bptr, lineatmp, &bsize, procstr);
					}
				}

				/* ver si hay que usar base64 */
				usebase64=!StrCmp(thisConTraEnc,"base64");

				if(!GetTokenParam(thisContentType,"charset=",thisCharset))
					strcpy(thisCharset,"us-ascii");
			}
			else
			{
				GetTokenValue(lineatmp,X_MimeContentType,thisContentType);
				GetTokenValue(lineatmp,X_MimeConTraEnc,thisConTraEnc);
				GetTokenValue(lineatmp,X_MimeConDes,thisDescription);
				GetTokenValue(lineatmp,MimeContentType,thisContentType);
				GetTokenValue(lineatmp,MimeConTraEnc,thisConTraEnc);
				GetTokenValue(lineatmp,MimeConDes,thisDescription);
			}

			continue;
		}

		/* si es multipart, comprueba si es un cambio */
		if(*thisBoundary)
		{
			if(lineatmp[0]=='-' && lineatmp[1]=='-')
			{
				if(begins(&lineatmp[2], thisBoundary)==0)
				{
					/* cambio de objeto */
					if(todisk>0)
                                        {
						close(todisk);
                                                todisk=0;

						/* si no tiene descripcion, se le
						   pone una */
						if(thisDescription[0]=='\0')
							sprintf(thisDescription,
							"%s, mensaje %d de %s",thisContentType,
							Msg.num,Conferencias[Conferencia]);

						/* aade la descripcion */
						Description(thisFileName);

						chdir("..");
					}

					/* solicita que se vuelvan a tomar el
					   content-type y el content-transfer-encoding */
					reload=1;

					/* si existe una funcion multipart, se la
					   llama ahora */
					if(multipart)
						multipart();

					/* valores por defecto */
					strcpy(thisContentType, "text/plain; charset=us-ascii");
					strcpy(thisConTraEnc, "8bit");

                                        continue;
				}
                        }
		}

		/* si est en quoted-printable, se decodifica */
		if(usequotprin)
		{
			DecodeQuotPrin(decodedline,lineatmp);
                        strcpy(lineatmp,decodedline);
		}

                /* si se esta procesando un objeto... */
                if(todisk>0)
                {
			if(usebase64)
			{
                        	DecodeBase64(decodedline, lineatmp, &n);
                        	write(todisk, decodedline, n);
			}
			else
			if(useuuencode)
			{
				if(begins(lineatmp,"end")==0)
				{
					close(todisk);
					useuuencode=0;

					sprintf(thisDescription, "uuencode, mensaje %d de %s",
						Msg.num, Conferencias[Conferencia]);

					Description(thisFileName);
				}
				else
				{
					DecodeUudecode(decodedline,lineatmp,&n);
                                        write(todisk,decodedline,n);
                                }
                        }
			else
			{
                        	/* es texto */
                        	write(todisk,lineatmp,strlen(lineatmp));
				write(todisk,NL_STR,sizeof(NL_STR)-1);
			}
                }
                else
                {
                        /* mira si hay un uuencode */
			if(begins(lineatmp,"begin ")==0 &&
				IS_NUM(lineatmp[6]) && IS_NUM(lineatmp[7]) &&
				IS_NUM(lineatmp[8]))
			{
                        	strcpy(thisFileName,&lineatmp[10]);

				NormFileName(thisFileName);

				chdir(FILES_DIR);

				todisk=open(thisFileName,
					O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,
					S_IREAD|S_IWRITE);

				/* aade una marca en el mensaje */
				if(todisk!=-1 && !quote)
                                {
					sprintf(lineatmp,"%s** Objeto uuencode: %s%s",
						NL_STR,thisFileName,NL_STR);
					bptr=Cpy(bptr, lineatmp, &bsize, procstr);
				}

                        	useuuencode=1;
			}
                        else
                        {
				/* anade el quote */
                                if(quote)
                                       	bptr=Cpy(bptr, QUOTE_CHARS, &bsize, NULL);

				bptr=Cpy(bptr, lineatmp, &bsize, procstr);
			}
                }
        }

        if(bptr)
                *bptr='\0';

        fclose(f);

        if(todisk)
        {
		close(todisk);
                todisk=0;
                chdir("..");
        }

	if(l)
		return(-1);
        else
		return(1);
}


int ReadMsg(char * hptr, int hsize, char * bptr, int bsize,
	void (* procstr)(char * cadena),
	void (* multipart) (void))
/* lee el mensaje usando ProcessMsg */
{
	return(ProcessMsg(hptr,hsize,bptr,bsize,procstr,0,multipart));
}


int QuoteMsg(char * bptr, int bsize, void (* procstr)(char * cadena))
/* rellena ptr con el mensaje actual marcado con > para edicion.
   Tambien modifica el subject. procstr es como en ReadMsg */
{
        int n,c,t;
        char tmp[512];
        int ok;

        if(!ProcessMsg(NULL,(int) NULL,bptr,bsize,procstr,1,NULL))
                return(0);

        /* se salta los espacios iniciales en el subject */
        for(c=0;Subject[c]==' ';c++);

        /* anade el Re: */
        if(strncmp(&Subject[c],"Re:",3)==0 ||
           strncmp(&Subject[c],"RE:",3)==0)
        {
                c+=4;

                if(Subject[c]=='(')
                {
                        /* salta el ( */
                        c++;

                        for(n=0;;n++,c++)
                        {
                                if(Subject[c]>='0' && Subject[c]<='9')
                                        tmp[n]=Subject[c];
                                else
                                        break;
                        }
                        tmp[n]=0;

                        t=atoi(tmp)+1;

                        /* salta el ) y el ' ' */
                        c+=2;
                }
                else
                        t=2;

                sprintf(tmp,"Re: (%d) %s", t, &Subject[c]);
        }
        else
                sprintf(tmp,"Re: %s",&Subject[c]);

        strcpy(Subject, tmp);

        return(1);
}


void WriteTag(int handle)
/* escribe un tag en el mensaje */
{
        FILE * f;
        long l;
        long l2;
        int c;
        int oc;
        char tag[2048];
        int n;

        /* si no existe, fuera */
        if((f=fopen(TAG_FILE, READ_BINARY))==0)
                return;

        /* coge el tamano maximo del fichero */
        fseek(f,0,SEEK_END);
        l=ftell(f);

        l2=(long) rand()|(((long) rand())<<16);
        l=l2%l;

        fseek(f,l,SEEK_SET);

        /* busca dos enters */
        for(oc=-1;(c=fgetc(f))!=EOF;oc=c)
        {
                if(c=='\r' && oc=='\n')
                        break;
        }

        /* se escribe en la cadena el tag */
        for(n=0,oc=-1;n<sizeof(tag)-1;n++,oc=c)
        {
                if((c=fgetc(f))==EOF)
                        break;

                if(c=='\r' && oc=='\n')
                        break;

                tag[n]=(char) c;
        }
        tag[n++]='\r';
        tag[n++]='\n';
        tag[n]=0;

        fclose(f);

        /* se escribe */
        write(handle, tag, strlen(tag));
}


void WordWrap(char * ptr)
/* inserta un \n en las palabras que se pasen */
{
        int x;
        char * sp;

        while(*ptr)
        {
                /* si es un quote... no se ajustan las palabras */
                if(*ptr=='>')
                {
                        while(*ptr!='\n' && *ptr!=0)
                                ptr++;
                }
                else
                {
                        /* bucle de lineas */
                        for(x=0,sp=0;*ptr!='\n' && *ptr!=0;ptr++,x++)
                        {
                                if(*ptr==' ' || *ptr=='\t')
                                        sp=ptr;

                                if(x>=ancho_linea)
                                {
                                        if(sp)
                                        {
                                                *sp='\n';
                                                ptr=sp+1;
                                                break;
                                        }
                                }
                        }

                        if(*ptr=='\n')
                                ptr++;
                }
        }
}


/** MIME **/

void DefaultMIME_EXT(void)
/* crea el fichero de asociaciones MIME por defecto */
{
	FILE * f;

	if((f=fopen(MIME_EXT,"r"))!=NULL)
	{
		fclose(f);
		return;
	}

	if((f=fopen(MIME_EXT,"w"))!=NULL)
	{
		fprintf(f,"text/plain|.txt,.doc,.c|%s", NL_STR);
                fprintf(f,"text/html|.htm,.html|%s", NL_STR);
		fprintf(f,"image/gif|.gif|%s", NL_STR);
		fprintf(f,"image/jpeg|.jpg,.jpeg|%s", NL_STR);
		fprintf(f,"image/tiff|.tif,.tiff|%s", NL_STR);
		fprintf(f,"image/x-bmp|.bmp|%s", NL_STR);
		fprintf(f,"audio/x-wav|.wav|%s", NL_STR);
		fprintf(f,"audio/midi|.mid,.midi|%s", NL_STR);
                fprintf(f,"video/mpeg|.mpg,.mpeg|%s", NL_STR);
		fprintf(f,"application/octet-stream||%s", NL_STR);

		fclose(f);
	}
}


void SearchMIME_EXT(char * content, char * ext, char * app)
/* busca la asociacion completa MIME */
{
	FILE * f;
	char * mimecontent;
	char * mimeext;
        char * mimeapp;

       	DefaultMIME_EXT();

	if((f=fopen(MIME_EXT, "r"))!=NULL)
	{
		for(;;)
                {
			if((fgets(lineatmp, sizeof(lineatmp)-1, f))==NULL)
			{
				if(content && content[0]=='\0')
					strcpy(content,"application/octet-stream");
				if(ext && ext[0]=='\0')
					strcpy(ext,"");
				if(app && app[0]=='\0')
					strcpy(app,"*");

                        	break;
                        }

			lineatmp[strlen(lineatmp)-1]='\0';

			if(lineatmp[0]=='[' || lineatmp[0]==';' ||
			   lineatmp[0]=='\0')
				continue;

			mimecontent=strtok(lineatmp,"|");
			mimeext=strtok(NULL,"|");
			mimeapp=strtok(NULL,"|");

			if(content[0]!='\0')
			{
				if(StrCmp(content,mimecontent)==0)
                                {
                        		if(ext)
						strcpy(ext, mimeext);

                                	if(app)
						strcpy(app, mimeapp);

                                	break;
                                }
			}
			else
			if(ext[0]!='\0')
			{
				mimeext=strtok(mimeext,",");

				while(mimeext)
                                {
					if(StrCmp(ext,mimeext)==0)
                                	{
						if(content)
							strcpy(content, mimecontent);

                                		if(app)
							strcpy(app, mimeapp);

						break;
					}

                                        mimeext=strtok(NULL,",");
                                }

				if(mimeext)
					break;
			}
			else
			if(app[0]!='\0')
			{
				if(StrCmp(app,mimeapp)==0)
                                {
					if(ext)
						strcpy(ext, mimeext);

                                	if(content)
						strcpy(content, mimecontent);

					break;
				}
			}
		}

		fclose(f);
	}
}

 
int WriteMIMEObject(int o, char * objfile, char * objdesc)
/* escribe objfile como un objeto MIME */
{
	int n;
        int astext;
        FILE * f;
	char linetocode[65];
        char ext[50];
        char filename[50];
	char content_type[50];

	if((f=fopen(objfile, READ_BINARY))==NULL)
		return(0);

	/* busca la extensin */
	for(n=strlen(objfile)-1;n>0;n--)
	{
		if(objfile[n]=='.')
			break;
	}

	if(n)
        {
		strcpy(ext, &objfile[n]);

		for(;n>0;n--)
		{
			if(objfile[n]=='\\' || objfile[n]=='/')
			{
                        	n++;
				break;
			}
		}
	}
	else
		strcpy(ext,".");

	/* guarda el nombre del fichero */
	strcpy(filename, &objfile[n]);

	StrLwr(ext);

	content_type[0]='\0';
        SearchMIME_EXT(content_type, ext, NULL);

	astext=!begins(content_type,"text/");

	if(astext)
	{
		sprintf(lineatmp,
		"--%s%s%s%s; charset=us-ascii; name=%s%s"\
		"%s8bit%s"\
		"%s%s%s%s",
		WailerBoundary,NL_STR,X_MimeContentType,content_type,filename,NL_STR,
		X_MimeConTraEnc,NL_STR,X_MimeConDes,objdesc,NL_STR,NL_STR);

		write(o,lineatmp,strlen(lineatmp));

		/* escribe todo el fichero */
		for(;;)
		{
			if(fgets(lineatmp,sizeof(lineatmp)-1,f)==NULL)
				break;

			write(o,lineatmp,strlen(lineatmp));
		}
	}
	else
	{ 
		sprintf(lineatmp,
		"--%s%s%s%s; name=%s%s"\
		"%sbase64%s"\
		"%s%s%s%s",
		WailerBoundary,NL_STR,X_MimeContentType,content_type,
		filename,NL_STR,
		X_MimeConTraEnc,NL_STR,
		X_MimeConDes,objdesc,NL_STR,NL_STR);

		write(o,lineatmp,strlen(lineatmp));

		/* escribe todo el fichero */
		for(;;)
		{
			n=fread(linetocode,1,57,f);

			if(n==0)
                        	break;

                        /* codifica en base64 */
			EncodeBase64(lineatmp,linetocode,n);

			/* anade el fin de linea */
			strcat(lineatmp, NL_STR);

			write(o,lineatmp,strlen(lineatmp));
		}
	}

	/* escribe el boundary final */
	sprintf(lineatmp,
		"--%s--%s%s", WailerBoundary,NL_STR,NL_STR);
	write(o,lineatmp,strlen(lineatmp));

	return(1);
}


int WriteMsg(char * bptr, char * conf, char * objfile, char * objdesc)
/* Envia el mensaje. Los datos se toman de las cadenas globales.
   El contenido de bptr se destruye si existe wailer.fir.
   Devuelve 0 si hubo error */
{
        int i,o;
        time_t t;
        struct tm * ti;

        /* abre el fichero */
        if((o=open(RESP_FILE,
                O_CREAT|O_BINARY|O_WRONLY,S_IREAD|S_IWRITE))==-1)
                return(0);

        /* va al final */
        lseek(o,0,SEEK_END);

	if(StrCmp(conf,"guest")==0)
		conf="contactos";

        /* conf se recibe porque puede no ser la conferencia actual */
        sprintf(lineatmp,"%sCONFERENCIA: %s%s",NL_STR,conf,NL_STR);
        write(o,lineatmp,strlen(lineatmp));

        sprintf(lineatmp,"Tema: %s%s",Subject, NL_STR);
        write(o,lineatmp,strlen(lineatmp));

        sprintf(lineatmp,"A: %s%s",To,NL_STR);
        write(o,lineatmp,strlen(lineatmp));

        /* se escribe el Message-ID */
        time(&t);
        ti=localtime(&t);
        sprintf(lineatmp,
                "Message-ID: %s_wailer_%s@crcok_%02d%02d%02d_%02d%02d%02d%s",
                Yo, conf, ti->tm_sec, ti->tm_min,
                ti->tm_hour, ti->tm_mday, ti->tm_mon+1, ti->tm_year, NL_STR);
        write(o,lineatmp,strlen(lineatmp));

        /* identificador de cabecera MIME */
	sprintf(lineatmp, "X-MIME-Version: 1.0 (%s)%s", szApp, NL_STR);
	write(o,lineatmp,strlen(lineatmp));

	/* si hay objfile, es multipart/mixed; si no, es text/plain */
	if(objfile==NULL)
	{
        	sprintf(lineatmp,
		"%stext/plain; charset=us-ascii%s"\
		"%s8bit%s",
		X_MimeContentType, NL_STR, X_MimeConTraEnc, NL_STR);
	}
	else
	{
		sprintf(lineatmp,
		"%smultipart/mixed; boundary=%s%s",
		X_MimeContentType, WailerBoundary,NL_STR);
	}

        write(o,lineatmp,strlen(lineatmp));

	/* Version del Newsreader: szApp esta definido fuera */
        sprintf(lineatmp,"X-Newsreader: %s%s", szApp,NL_STR);
        write(o,lineatmp,strlen(lineatmp));

        /* linea de separacion entre cabecera y cuerpo */
	sprintf(lineatmp,"%s",NL_STR);
	write(o,lineatmp,strlen(lineatmp));

        /* se cortan las palabras */
        WordWrap(bptr);

	/* si hay objeto, se inserta la cabeza del mensaje */
	if(objfile!=NULL)
	{
		sprintf(lineatmp,
		"--%s%s%stext/plain; charset=us-ascii%s"\
		"%s8bit%s%s",
		WailerBoundary,NL_STR,X_MimeContentType,NL_STR,
		X_MimeConTraEnc,NL_STR,NL_STR);

		write(o,lineatmp,strlen(lineatmp));
	}

        /* se escribe el mensaje */
        write(o,bptr,strlen(bptr));

        /* si hay fichero de firma se anade */
        if((i=open(FIRM_FILE,O_RDONLY|O_BINARY))!=-1)
        {
                write(o,bptr,read(i,bptr,32000));
                close(i);
        }

	sprintf(lineatmp,"%s%s-- %s%s%s", NL_STR, NL_STR,szApp, NL_STR,NL_STR);
        write(o, lineatmp, strlen(lineatmp));

        WriteTag(o);

        /* anade el fichero pegado */
	if(objfile!=NULL)
		WriteMIMEObject(o, objfile, objdesc);

        /* lo cierra */
        close(o);

        return(1);
}


/** funciones de lista de mensajes **/

int ReadList(void (* newelem)(void))
/* Procesa la lista de mensajes de la conferencia actual,
   rellenando el fichero temporal de offsets
   y llamando a newelem cada vez que se lea un mensaje. Cuando se
   llame a newelem se habran rellenado Subject, Remit y To */
{
        int i,d,t;
        struct tagind ti;
	int n;
	int num;

        NumMsgs=num=0;

        /* si no hay conferencia... */
        if(Conferencia==-1)
                return(0);

        /* se abren los ficheros */
        if(Histo==1)
        {
                i=open(HISTO_IND, O_RDONLY|O_BINARY);
                d=open(HISTO_DAT, O_RDONLY|O_BINARY);
        }
        else
        {
                i=open(NUEVOS_IND, O_RDONLY|O_BINARY);
                d=open(NUEVOS_DAT, O_RDONLY|O_BINARY);
        }

        if(i==-1 || d==-1)
        {
                close(i);
                close(d);
                return(0);
        }

        /* se crea el fichero temporal */
        if((t=open(TMP_FILE, O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,
                S_IREAD|S_IWRITE))==-1)
        {
                close(i);
                close(d);
                return(0);
        }

        /* bucle de lectura del indice */
        for(;;)
        {
                /* se lee un tag del indice */
                if(read(i, &ti, sizeof(ti))!=sizeof(ti))
                        break;

                /* si no es de esta conferencia, adios */
                if((ti.dat&MASK_CONF)!=Conferencia)
                {
                        num++;
                        continue;
                }

                /* si esta borrado y es el historico, no sale */
                if((ti.dat&MASK_BOR) && Histo)
                {
                        num++;
                        continue;
                }

                /* si este mensaje vale, se guarda en el temporal */
                write(t,&num, sizeof(int));

                if(ti.offsub!=0xFFFF)
                {
                        /* se lee el subject */
                        lseek(d, (long)(ti.offset+ti.offsub), 0);
                        read(d, Subject, sizeof(Subject)-1);
                }
                else
                        Subject[0]=0;

                /* quita caracteres sobrantes */
                for(n=0;Subject[n]!='\n' && Subject[n]!=0 &&
                        n<sizeof(Subject)-1;n++);
                Subject[n]=0;

                if(ti.offfrom!=0xFFFF)
                {
                        /* se lee el from */
                        lseek(d, (long)(ti.offset+ti.offfrom), 0);
                        read(d, Remit, sizeof(Remit)-1);
                }
                else
                        Remit[0]=0;

                GetLogName(Remit);

                if(ti.offto!=0xFFFF)
                {
                        /* se lee el to */
                        lseek(d, (long)(ti.offset+ti.offto), 0);
                        read(d, To, sizeof(To)-1);
                }
                else
                        To[0]=0;

                GetLogName(To);

                num++;
                NumMsgs++;

		/* si existe una funcion de proceso, se la llama */
		if(newelem)
                        (*newelem)();
        }

        close(i);
        close(d);
        close(t);

        return(1);
}


/** funciones de ficheros **/

int Index(char * dat, char * ind)
/* crea un indice .ind segun el fichero .dat */
{
        FILE * nd;
        struct tagind ti;
        int i;
        char tmp[50];
        int n,m;
        char firstline[100];
        long l;
        char * ptr;
        int saveit;

        /* abre los datos y crea el indice */
        if((nd=fopen(dat, READ_BINARY))==NULL)
                return(0);

        if((i=open(ind, O_CREAT|O_TRUNC|O_WRONLY|O_BINARY,
                S_IREAD|S_IWRITE))==-1)
        {
                fclose(nd);
                return(0);
        }

	/* se salta las posibles lineas en blanco al principio */
	for(;;)
        {
        	l=ftell(nd);
        	fgets(lineatmp, sizeof(lineatmp)-1, nd);

		if(lineatmp[0]!='\0' && lineatmp[0]!='\n')
		{
			fseek(nd, l, 0);
			break;
		}
	}

       	lineatmp[0]=' ';
	lineatmp[1]=0;

        /* lee la primera linea con informacion, para averiguar
           cual es la primera linea de cada mensaje */
        l=ftell(nd);
        fgets(firstline, sizeof(firstline)-1, nd);
        ptr=strchr(firstline, ':');
        if(ptr)
                *ptr=0;
        fseek(nd, l, 0);

        ti.offset=1;
        ti.tamcab=0;
        ti.tambody=0;

        Stats.mensajes=Stats.personales=0;

        for(;;)
        {
                /* se definen los valores iniciales */
                ti.dat=0;
                ti.offset+=ti.tamcab+ti.tambody;
                ti.tamcab=ti.tambody=0;

                ti.offsub=ti.offfrom=ti.offto=0xFFFF;

		saveit=1;

                /* lee una cabecera */
                for(;;)
                {
                        if(feof(nd))
                                break;

                        /* si la linea no contiene nada,
                           se acabo la cabecera */
                        if(lineatmp[0]==0 || lineatmp[0]=='\n')
                                break;

                        /* compara tokens */
                        if(begins(lineatmp, "Mensaje ")==0)
                        {
                                /* se coge el numero de mensaje */
                                for(n=0,m=8;lineatmp[m]!=' ';n++,m++)
                                        tmp[n]=lineatmp[m];

                                tmp[n]=0;

                                ti.num=atoi(tmp);
                        }
                        else
                        if(begins(lineatmp, firstline)==0 ||
			   begins(lineatmp, "X-BBS:")==0 ||
                           begins(lineatmp, "X-MMedia:")==0)
                        {
                                /* la linea es X-BBS: <conferencia> [mensaje] */
                                /* por tanto, se busca el < */
                                for(n=0;lineatmp[n]!='<' && lineatmp[n];n++);

                                if(lineatmp[n])
                                {
                                        n++;

                                        /* se lee la conferencia. Antes se leia
                                           de Newsgroups:, pero daba problemas
                                           en las news */
                                        for(m=0;lineatmp[n]!='>' && lineatmp[n];n++,m++)
                                                tmp[m]=lineatmp[n];
                                        tmp[m]=0;

                                        ti.dat=(unsigned char) GetConf(tmp);
                                }

                                /* busca el numero de mensaje */
                                for(n++;lineatmp[n]!='[' && lineatmp[n];n++);

                                if(lineatmp[n])
                                {
                                        n++;

                                        /* se escribe en tmp el numero */
                                        for(m=0;lineatmp[n]!=']' && lineatmp[n];n++,m++)
                                                tmp[m]=lineatmp[n];
                                        tmp[m]=0;

                                        ti.num=atoi(tmp);
                                }
                                else
                                        ti.num=0;
                        }
                        else
                        if(begins(lineatmp, "From: ")==0)
                        {
                                /* se define el offset del from */
				ti.offfrom=ti.tamcab+5;

				strcpy(Remit,&lineatmp[6]);
				GetLogName(Remit);

				/* mira si es un indeseable */
				for(ptr=KillFile;ptr!=NULL && *ptr;)
				{
					if(begins(ptr,Remit)==0)
					{
                                        	saveit=0;
						break;
					}

					ptr=strchr(ptr,',');

					if(ptr)
						ptr++;
                                }
			}
                        else
                        if(begins(lineatmp, "To: ")==0)
                        {
                                /* se define el offset del to */
                                ti.offto=ti.tamcab+3;

				if(begins(lineatmp+4,Yo)==0)
                                {
                                        Stats.personales++;
                                }
                        }
                        else
                        if(begins(lineatmp, "Subject: ")==0)
                        {
                                /* se define el offset del subject */
                                ti.offsub=ti.tamcab+7;
                        }

                        /* se anade longitud a la cabecera */
                        ti.tamcab+=strlen(lineatmp);

                        /* lee una linea */
                        fgets(lineatmp, sizeof(lineatmp)-1, nd);
                }

                if(feof(nd))
                        break;

                ti.tamcab+=strlen(lineatmp);

                for(;;)
                {
                        /* lee una linea */
                        fgets(lineatmp, sizeof(lineatmp)-1, nd);

                        if(feof(nd))
                                break;

			if(begins(lineatmp, "X-BBS:")==0 ||
                           begins(lineatmp, "X-MMedia:")==0 ||
                           begins(lineatmp, firstline)==0)
                                break;

                        ti.tambody+=strlen(lineatmp);
                }

                /* escribe el tag del fichero */
		if(saveit)
                {
                	write(i, &ti, sizeof(ti));

			Stats.mensajes++;
                }

                if(feof(nd))
                        break;
        }

	fclose(nd);
        close(i);

        return(1);
}


int DropMsg(void)
/* pasa al historico los mensajes nuevos que se quieren guardar */
{
        int hd,hi,ni,nd;
        struct tagind ti;
        char * ptr;
	int n;
	long t;

        /* se abre el indice de nuevos */
        if((ni=open(NUEVOS_IND,O_RDONLY|O_BINARY))==-1)
                return(0);

        /* datos de nuevos */
        if((nd=open(NUEVOS_DAT,O_RDONLY|O_BINARY))==-1)
        {
                close(ni);
                return(0);
        }

        /* indice al historico */
        if((hi=open(HISTO_IND,O_CREAT|O_BINARY|O_WRONLY,S_IREAD|S_IWRITE))==-1)
        {
                close(ni);
                close(nd);
                return(0);
        }

        /* datos del historico */
        if((hd=open(HISTO_DAT,O_CREAT|O_BINARY|O_WRONLY,S_IREAD|S_IWRITE))==-1)
        {
                close(ni);
                close(nd);
                close(hi);
                return(0);
        }

        /* se va al final del indice y del historico */
        lseek(hi, 0, SEEK_END);
        lseek(hd, 0, SEEK_END);

        for(;;)
        {
                /* lee un tag del indice */
                if(read(ni, &ti, sizeof(ti))!=sizeof(ti))
                        break;

                /* si no esta borrado */
                if(!(ti.dat&MASK_BOR))
                {
                        /* se posiciona en el mensaje */
                        lseek(nd, ti.offset-1, 0);

                        /* se coge el nuevo offset del historico */
                        ti.offset=tell(hd);

                        /* se escribe el cuerpo del mensaje */
			for(t=ti.tamcab+ti.tambody;t;t-=(long) n)
                        {
				if(t>sizeof(lineatmp)-1)
					n=sizeof(lineatmp)-1;
				else
                                	n=(int) t;

                                n=read(nd, lineatmp, n);

                                write(hd, lineatmp, n);
                        }

                        /* se escribe en el indice historico */
                        write(hi, &ti, sizeof(ti));
                }
        }

        close(ni);
        close(nd);
        close(hi);
        close(hd);

        return(1);
}


/** funciones de inicializacion **/

void ProcessNew(void)
/* procesa los nuevos mensajes */
{
        /* copia los no borrados al historico */
        if(use_histo)
                DropMsg();

        /* borra los nuevos */
        unlink(NUEVOS_DAT);

        /* renombra los mensajes a los nuevos */
        rename(MSG_FILE, NUEVOS_DAT);

        /* crea el indice de nuevos */
	Index(NUEVOS_DAT, NUEVOS_IND);
}


int IsRespFile(void)
/* mira si existe un fichero de respuestas */
{
        int i;

        if((i=open(RESP_FILE, O_RDONLY|O_BINARY))==-1)
                return(0);
        else
        {
                close(i);
                return(1);
        }
}


void InitRespFile(void)
/* inicializa el fichero de respuestas */
{
        /* simplemente lo borra */
        unlink(RESP_FILE);
}


int InitW(void)
/* inicializa el sistema. Devuelve:
   1 si no hay un paquete nuevo de mensajes.
   0 si no lo hay. */
{
        int i;

        /* aleatoriza random */
        srand((unsigned int) time(NULL));

	/* crea el directorio de ficheros */
#if defined(UNIX_HOST)
	mkdir(FILES_DIR, 0777);
#else
	mkdir(FILES_DIR);
#endif

	/* se cargan las conferencias */
        LoadConf();

	/* si no existe un fichero de mensajes... */
        if((i=open(MSG_FILE,O_RDONLY|O_BINARY))==-1)
                return(0);

        close(i);

        return(1);
}


void EndW(void)
/* finaliza */
{
        /* borra el fichero temporal */
        unlink(TMP_FILE);
}

